@@ -19,11 +19,11 @@ from ..middleware import FiretailMiddleware
 from ..problem import problem
 from .abstract import AbstractApp
 
-logger = logging.getLogger('firetail.app')
+logger = logging.getLogger("firetail.app")
 
 
 class FlaskApp(AbstractApp):
-    def __init__(self, import_name, server='flask', extra_files=None, **kwargs):
+    def __init__(self, import_name, server="flask", extra_files=None, **kwargs):
         """
         :param extra_files: additional files to be watched by the reloader, defaults to the swagger specs of added apis
         :type extra_files: list[str | pathlib.Path], optional
@@ -37,15 +37,13 @@ class FlaskApp(AbstractApp):
     def create_app(self):
         app = flask.Flask(self.import_name, **self.server_args)
         app.json = FlaskJSONProvider(app)
-        app.url_map.converters['float'] = NumberConverter
-        app.url_map.converters['int'] = IntegerConverter
+        app.url_map.converters["float"] = NumberConverter
+        app.url_map.converters["int"] = IntegerConverter
         return app
 
-    def _apply_middleware(self):
-        middlewares = [*FiretailMiddleware.default_middlewares,
-                       a2wsgi.WSGIMiddleware]
-        middleware = FiretailMiddleware(
-            self.app.wsgi_app, middlewares=middlewares)
+    def _apply_middleware(self, middlewares):
+        middlewares = [*middlewares, a2wsgi.WSGIMiddleware]
+        middleware = FiretailMiddleware(self.app.wsgi_app, middlewares=middlewares)
 
         # Wrap with ASGI to WSGI middleware for usage with development server and test client
         self.app.wsgi_app = a2wsgi.ASGIMiddleware(middleware)
@@ -68,17 +66,24 @@ class FlaskApp(AbstractApp):
         signals.got_request_exception.send(self.app, exception=exception)
         if isinstance(exception, ProblemException):
             response = problem(
-                status=exception.status, title=exception.title, detail=exception.detail,
-                type=exception.type, instance=exception.instance, headers=exception.headers,
-                ext=exception.ext)
+                status=exception.status,
+                title=exception.title,
+                detail=exception.detail,
+                type=exception.type,
+                instance=exception.instance,
+                headers=exception.headers,
+                ext=exception.ext,
+            )
         else:
             if not isinstance(exception, werkzeug.exceptions.HTTPException):
                 exception = werkzeug.exceptions.InternalServerError()
 
-            response = problem(title=exception.name,
-                               detail=exception.description,
-                               status=exception.code,
-                               headers=exception.get_headers())
+            response = problem(
+                title=exception.name,
+                detail=exception.description,
+                status=exception.code,
+                headers=exception.get_headers(),
+            )
 
         return FlaskApi.get_response(response)
 
@@ -93,13 +98,9 @@ class FlaskApp(AbstractApp):
         # type: (int, FunctionType) -> None
         self.app.register_error_handler(error_code, function)
 
-    def run(self,
-            port=None,
-            server=None,
-            debug=None,
-            host=None,
-            extra_files=None,
-            **options):  # pragma: no cover
+    def run(
+        self, port=None, server=None, debug=None, host=None, extra_files=None, **options
+    ):  # pragma: no cover
         """
         Runs the application on a local development server.
 
@@ -123,7 +124,7 @@ class FlaskApp(AbstractApp):
         elif self.port is None:
             self.port = 5000
 
-        self.host = host or self.host or '0.0.0.0'
+        self.host = host or self.host or "0.0.0.0"
 
         if server is not None:
             self.server = server
@@ -134,35 +135,46 @@ class FlaskApp(AbstractApp):
         if extra_files is not None:
             self.extra_files.extend(extra_files)
 
-        logger.debug('Starting %s HTTP server..',
-                     self.server, extra=vars(self))
-        if self.server == 'flask':
-            self.app.run(self.host, port=self.port, debug=self.debug,
-                         extra_files=self.extra_files, **options)
-        elif self.server == 'tornado':
+        logger.debug("Starting %s HTTP server..", self.server, extra=vars(self))
+        if self.server == "flask":
+            self.app.run(
+                self.host,
+                port=self.port,
+                debug=self.debug,
+                extra_files=self.extra_files,
+                **options,
+            )
+        elif self.server == "tornado":
             try:
+                import tornado.autoreload
                 import tornado.httpserver
                 import tornado.ioloop
                 import tornado.wsgi
             except ImportError:
-                raise Exception('tornado library not installed')
+                raise Exception("tornado library not installed")
             wsgi_container = tornado.wsgi.WSGIContainer(self.app)
-            http_server = tornado.httpserver.HTTPServer(
-                wsgi_container, **options)
+            http_server = tornado.httpserver.HTTPServer(wsgi_container, **options)
             http_server.listen(self.port, address=self.host)
-            logger.info('Listening on %s:%s..', self.host, self.port)
+            if self.debug:
+                tornado.autoreload.start()
+            logger.info("Listening on %s:%s..", self.host, self.port)
             tornado.ioloop.IOLoop.instance().start()
-        elif self.server == 'gevent':
+        elif self.server == "gevent":
             try:
                 import gevent.pywsgi
             except ImportError:
-                raise Exception('gevent library not installed')
+                raise Exception("gevent library not installed")
+            if self.debug:
+                logger.warning(
+                    "gevent server doesn't support debug mode. Please switch to flask/tornado server."
+                )
             http_server = gevent.pywsgi.WSGIServer(
-                (self.host, self.port), self.app, **options)
-            logger.info('Listening on %s:%s..', self.host, self.port)
+                (self.host, self.port), self.app, **options
+            )
+            logger.info("Listening on %s:%s..", self.host, self.port)
             http_server.serve_forever()
         else:
-            raise Exception(f'Server {self.server} not recognized')
+            raise Exception(f"Server {self.server} not recognized")
 
     def __call__(self, scope, receive, send):  # pragma: no cover
         """
@@ -180,7 +192,8 @@ class FlaskJSONProvider(flask.json.provider.DefaultJSONProvider):
 
 
 class NumberConverter(werkzeug.routing.BaseConverter):
-    """ Flask converter for OpenAPI number type """
+    """Flask converter for OpenAPI number type"""
+
     regex = r"[+-]?[0-9]*(\.[0-9]*)?"
 
     def to_python(self, value):
@@ -188,7 +201,8 @@ class NumberConverter(werkzeug.routing.BaseConverter):
 
 
 class IntegerConverter(werkzeug.routing.BaseConverter):
-    """ Flask converter for OpenAPI integer type """
+    """Flask converter for OpenAPI integer type"""
+
     regex = r"[+-]?[0-9]+"
 
     def to_python(self, value):
