@@ -5,7 +5,7 @@ This module defines Exception classes used by Firetail to generate a proper resp
 import warnings
 
 from jsonschema.exceptions import ValidationError
-from werkzeug.exceptions import Forbidden, Unauthorized
+from starlette.exceptions import HTTPException
 
 from .problem import problem
 
@@ -15,8 +15,16 @@ class FiretailException(Exception):
 
 
 class ProblemException(FiretailException):
-    def __init__(self, status=400, title=None, detail=None, type=None,
-                 instance=None, headers=None, ext=None):
+    def __init__(
+        self,
+        status=400,
+        title=None,
+        detail=None,
+        type=None,
+        instance=None,
+        headers=None,
+        ext=None,
+    ):
         """
         This exception holds arguments that are going to be passed to the
         `firetail.problem` function to generate a proper response.
@@ -32,14 +40,22 @@ class ProblemException(FiretailException):
     def to_problem(self):
         warnings.warn(
             "'to_problem' is planned to be removed in a future release. "
-            "Call firetail.problem.problem(..) instead to maintain the existing error response.", DeprecationWarning)
-        return problem(status=self.status, title=self.title, detail=self.detail,
-                       type=self.type, instance=self.instance, headers=self.headers,
-                       ext=self.ext)
+            "Call firetail.problem.problem(..) instead to maintain the existing error response.",
+            DeprecationWarning,
+        )
+        return problem(
+            status=self.status,
+            title=self.title,
+            detail=self.detail,
+            type=self.type,
+            instance=self.instance,
+            headers=self.headers,
+            ext=self.ext,
+        )
 
 
 class ResolverError(LookupError):
-    def __init__(self, reason='Unknown reason', exc_info=None):
+    def __init__(self, reason="Unknown reason", exc_info=None):
         """
         :param reason: Reason why the resolver failed.
         :type reason: str
@@ -51,18 +67,22 @@ class ResolverError(LookupError):
         self.exc_info = exc_info
 
     def __str__(self):  # pragma: no cover
-        return f'<ResolverError: {self.reason}>'
+        return f"<ResolverError: {self.reason}>"
 
     def __repr__(self):  # pragma: no cover
-        return f'<ResolverError: {self.reason}>'
+        return f"<ResolverError: {self.reason}>"
 
 
 class InvalidSpecification(FiretailException, ValidationError):
     pass
 
 
+class MissingMiddleware(FiretailException):
+    pass
+
+
 class NonConformingResponse(ProblemException):
-    def __init__(self, reason='Unknown Reason', message=None):
+    def __init__(self, reason="Unknown Reason", message=None):
         """
         :param reason: Reason why the response did not conform to the specification
         :type reason: str
@@ -72,46 +92,70 @@ class NonConformingResponse(ProblemException):
         self.message = message
 
     def __str__(self):  # pragma: no cover
-        return f'<NonConformingResponse: {self.reason}>'
+        return f"<NonConformingResponse: {self.reason}>"
 
     def __repr__(self):  # pragma: no cover
-        return f'<NonConformingResponse: {self.reason}>'
+        return f"<NonConformingResponse: {self.reason}>"
 
 
 class AuthenticationProblem(ProblemException):
-
     def __init__(self, status, title, detail):
         super().__init__(status=status, title=title, detail=detail)
 
 
 class ResolverProblem(ProblemException):
-
     def __init__(self, status, title, detail):
         super().__init__(status=status, title=title, detail=detail)
 
 
 class BadRequestProblem(ProblemException):
-
-    def __init__(self, title='Bad Request', detail=None):
+    def __init__(self, title="Bad Request", detail=None):
         super().__init__(status=400, title=title, detail=detail)
 
 
-class UnsupportedMediaTypeProblem(ProblemException):
+class NotFoundProblem(ProblemException):
 
+    description = (
+        "The requested URL was not found on the server. If you entered the URL manually please "
+        "check your spelling and try again."
+    )
+
+    def __init__(self, title="Not Found", detail=description):
+        super().__init__(status=404, title=title, detail=detail)
+
+
+class UnsupportedMediaTypeProblem(ProblemException):
     def __init__(self, title="Unsupported Media Type", detail=None):
         super().__init__(status=415, title=title, detail=detail)
 
 
 class NonConformingResponseBody(NonConformingResponse):
-    def __init__(self, message, reason="Response body does not conform to specification"):
+    def __init__(
+        self, message, reason="Response body does not conform to specification"
+    ):
         super().__init__(reason=reason, message=message)
 
 
 class NonConformingResponseHeaders(NonConformingResponse):
-    def __init__(self, message, reason="Response headers do not conform to specification"):
+    def __init__(
+        self, message, reason="Response headers do not conform to specification"
+    ):
         super().__init__(reason=reason, message=message)
 
 
+class Unauthorized(HTTPException):
+
+    description = (
+        "The server could not verify that you are authorized to access"
+        " the URL requested. You either supplied the wrong credentials"
+        " (e.g. a bad password), or your browser doesn't understand"
+        " how to supply the credentials required."
+    )
+
+    def __init__(self, detail: str = description, **kwargs):
+        super().__init__(401, detail=detail, **kwargs)
+
+
 class OAuthProblem(Unauthorized):
     pass
 
@@ -122,6 +166,18 @@ class OAuthResponseProblem(OAuthProblem):
         super().__init__(**kwargs)
 
 
+class Forbidden(HTTPException):
+
+    description = (
+        "You don't have the permission to access the requested"
+        " resource. It is either read-protected or not readable by the"
+        " server."
+    )
+
+    def __init__(self, detail: str = description, **kwargs):
+        super().__init__(403, detail=detail, **kwargs)
+
+
 class OAuthScopeProblem(Forbidden):
     def __init__(self, token_scopes, required_scopes, **kwargs):
         self.required_scopes = required_scopes
@@ -131,17 +187,22 @@ class OAuthScopeProblem(Forbidden):
 
 
 class ExtraParameterProblem(ProblemException):
-    def __init__(self, formdata_parameters, query_parameters, title=None, detail=None, **kwargs):
+    def __init__(
+        self, formdata_parameters, query_parameters, title=None, detail=None, **kwargs
+    ):
         self.extra_formdata = formdata_parameters
         self.extra_query = query_parameters
 
         # This keep backwards compatibility with the old returns
         if detail is None:
             if self.extra_query:
-                detail = "Extra {parameter_type} parameter(s) {extra_params} not in spec"\
-                    .format(parameter_type='query', extra_params=', '.join(self.extra_query))
+                detail = "Extra {parameter_type} parameter(s) {extra_params} not in spec".format(
+                    parameter_type="query", extra_params=", ".join(self.extra_query)
+                )
             elif self.extra_formdata:
-                detail = "Extra {parameter_type} parameter(s) {extra_params} not in spec"\
-                    .format(parameter_type='formData', extra_params=', '.join(self.extra_formdata))
+                detail = "Extra {parameter_type} parameter(s) {extra_params} not in spec".format(
+                    parameter_type="formData",
+                    extra_params=", ".join(self.extra_formdata),
+                )
 
         super().__init__(title=title, detail=detail, **kwargs)
