@@ -3,15 +3,16 @@ This module defines an abstract SecurityHandlerFactory which supports the creati
 handlers for operations.
 """
 
-import abc
+import asyncio
 import base64
-import functools
 import http.cookies
 import logging
 import os
 import textwrap
 import typing as t
 
+import httpx
+
 from ..decorators.parameter import inspect_function_arguments
 from ..exceptions import (
     FiretailException,
@@ -21,10 +22,10 @@ from ..exceptions import (
 )
 from ..utils import get_function_from_name
 
-logger = logging.getLogger('firetail.api.security')
+logger = logging.getLogger("firetail.api.security")
 
 
-class AbstractSecurityHandlerFactory(abc.ABC):
+class SecurityHandlerFactory:
     """
     get_*_func -> _get_function -> get_function_from_name (name=security function defined in spec)
         (if url defined instead of a function -> get_token_info_remote)
@@ -38,19 +39,22 @@ class AbstractSecurityHandlerFactory(abc.ABC):
 
         verify helpers (used inside wrappers): get_auth_header_value, get_cookie_value
     """
-    no_value = object()
-    required_scopes_kw = 'required_scopes'
 
-    def __init__(self, pass_context_arg_name):
-        self.pass_context_arg_name = pass_context_arg_name
+    no_value = object()
+    required_scopes_kw = "required_scopes"
+    context_kw = "context_"
+    client = None
 
     @staticmethod
-    def _get_function(security_definition, security_definition_key, environ_key, default=None):
+    def _get_function(
+        security_definition, security_definition_key, environ_key, default=None
+    ):
         """
         Return function by getting its name from security_definition or environment variable
         """
-        func = security_definition.get(
-            security_definition_key) or os.environ.get(environ_key)
+        func = security_definition.get(security_definition_key) or os.environ.get(
+            environ_key
+        )
         if func:
             return get_function_from_name(func)
         return default
@@ -63,12 +67,14 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         '<function foo.bar>'
         """
         token_info_func = self._get_function(
-            security_definition, "x-tokenInfoFunc", 'TOKENINFO_FUNC')
+            security_definition, "x-tokenInfoFunc", "TOKENINFO_FUNC"
+        )
         if token_info_func:
             return token_info_func
 
-        token_info_url = (security_definition.get('x-tokenInfoUrl') or
-                          os.environ.get('TOKENINFO_URL'))
+        token_info_url = security_definition.get("x-tokenInfoUrl") or os.environ.get(
+            "TOKENINFO_URL"
+        )
         if token_info_url:
             return self.get_token_info_remote(token_info_url)
 
@@ -83,7 +89,12 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         >>> get_scope_validate_func({'x-scopeValidateFunc': 'foo.bar'})
         '<function foo.bar>'
         """
-        return cls._get_function(security_definition, "x-scopeValidateFunc", 'SCOPEVALIDATE_FUNC', cls.validate_scope)
+        return cls._get_function(
+            security_definition,
+            "x-scopeValidateFunc",
+            "SCOPEVALIDATE_FUNC",
+            cls.validate_scope,
+        )
 
     @classmethod
     def get_basicinfo_func(cls, security_definition):
@@ -94,7 +105,9 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         >>> get_basicinfo_func({'x-basicInfoFunc': 'foo.bar'})
         '<function foo.bar>'
         """
-        return cls._get_function(security_definition, "x-basicInfoFunc", 'BASICINFO_FUNC')
+        return cls._get_function(
+            security_definition, "x-basicInfoFunc", "BASICINFO_FUNC"
+        )
 
     @classmethod
     def get_apikeyinfo_func(cls, security_definition):
@@ -105,7 +118,9 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         >>> get_apikeyinfo_func({'x-apikeyInfoFunc': 'foo.bar'})
         '<function foo.bar>'
         """
-        return cls._get_function(security_definition, "x-apikeyInfoFunc", 'APIKEYINFO_FUNC')
+        return cls._get_function(
+            security_definition, "x-apikeyInfoFunc", "APIKEYINFO_FUNC"
+        )
 
     @classmethod
     def get_bearerinfo_func(cls, security_definition):
@@ -116,15 +131,13 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         >>> get_bearerinfo_func({'x-bearerInfoFunc': 'foo.bar'})
         '<function foo.bar>'
         """
-        return cls._get_function(security_definition, "x-bearerInfoFunc", 'BEARERINFO_FUNC')
+        return cls._get_function(
+            security_definition, "x-bearerInfoFunc", "BEARERINFO_FUNC"
+        )
 
     @staticmethod
-    def security_passthrough(function):
-        """
-        :type function: types.FunctionType
-        :rtype: types.FunctionType
-        """
-        return function
+    async def security_passthrough(request):
+        return request
 
     @staticmethod
     def security_deny(function):
@@ -153,10 +166,15 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         logger.debug("... Scopes required: %s", required_scopes)
         logger.debug("... Token scopes: %s", token_scopes)
         if not required_scopes <= token_scopes:
-            logger.info(textwrap.dedent("""
+            logger.info(
+                textwrap.dedent(
+                    """
                         ... Token scopes (%s) do not match the scopes necessary to call endpoint (%s).
-                         Aborting with 403.""").replace('\n', ''),
-                        token_scopes, required_scopes)
+                         Aborting with 403."""
+                ).replace("\n", ""),
+                token_scopes,
+                required_scopes,
+            )
             return False
         return True
 
@@ -169,23 +187,22 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         If not Authorization, return (None, None)
         Raise OAuthProblem for invalid Authorization header
         """
-        authorization = request.headers.get('Authorization')
+        authorization = request.headers.get("Authorization")
         if not authorization:
             return None, None
 
         try:
             auth_type, value = authorization.split(None, 1)
         except ValueError:
-            raise OAuthProblem(description='Invalid authorization header')
+            raise OAuthProblem(detail="Invalid authorization header")
         return auth_type.lower(), value
 
     def verify_oauth(self, token_info_func, scope_validate_func, required_scopes):
-        check_oauth_func = self.check_oauth_func(
-            token_info_func, scope_validate_func)
+        check_oauth_func = self.check_oauth_func(token_info_func, scope_validate_func)
 
         def wrapper(request):
             auth_type, token = self.get_auth_header_value(request)
-            if auth_type != 'bearer':
+            if auth_type != "bearer":
                 return self.no_value
 
             return check_oauth_func(request, token, required_scopes=required_scopes)
@@ -197,14 +214,15 @@ class AbstractSecurityHandlerFactory(abc.ABC):
 
         def wrapper(request):
             auth_type, user_pass = self.get_auth_header_value(request)
-            if auth_type != 'basic':
+            if auth_type != "basic":
                 return self.no_value
 
             try:
-                username, password = base64.b64decode(
-                    user_pass).decode('latin1').split(':', 1)
+                username, password = (
+                    base64.b64decode(user_pass).decode("latin1").split(":", 1)
+                )
             except Exception:
-                raise OAuthProblem(description='Invalid authorization header')
+                raise OAuthProblem(detail="Invalid authorization header")
 
             return check_basic_info_func(request, username, password)
 
@@ -212,13 +230,13 @@ class AbstractSecurityHandlerFactory(abc.ABC):
 
     @staticmethod
     def get_cookie_value(cookies, name):
-        '''
+        """
         Called inside security wrapper functions
 
         Returns cookie value by its name. None if no such value.
         :param cookies: str: cookies raw data
         :param name: str: cookies key
-        '''
+        """
         cookie_parser = http.cookies.SimpleCookie()
         cookie_parser.load(str(cookies))
         try:
@@ -230,7 +248,6 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         check_api_key_func = self.check_api_key(api_key_info_func)
 
         def wrapper(request):
-
             def _immutable_pop(_dict, key):
                 """
                 Pops the key from an immutable dict and returns the value that was popped,
@@ -244,16 +261,15 @@ class AbstractSecurityHandlerFactory(abc.ABC):
                     _dict = dict(_dict.items())
                     return _dict.pop(key), cls(_dict)
 
-            if loc == 'query':
+            if loc == "query":
                 try:
-                    api_key, request.query = _immutable_pop(
-                        request.query, name)
+                    api_key, request.query = _immutable_pop(request.query, name)
                 except KeyError:
                     api_key = None
-            elif loc == 'header':
+            elif loc == "header":
                 api_key = request.headers.get(name)
-            elif loc == 'cookie':
-                cookie_list = request.headers.get('Cookie')
+            elif loc == "cookie":
+                cookie_list = request.headers.get("Cookie")
                 api_key = self.get_cookie_value(cookie_list, name)
             else:
                 return self.no_value
@@ -274,7 +290,7 @@ class AbstractSecurityHandlerFactory(abc.ABC):
 
         def wrapper(request):
             auth_type, token = self.get_auth_header_value(request)
-            if auth_type != 'bearer':
+            if auth_type != "bearer":
                 return self.no_value
             return check_bearer_func(request, token)
 
@@ -290,10 +306,12 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         :rtype: types.FunctionType
         """
 
-        def wrapper(request):
+        async def wrapper(request):
             token_info = {}
             for scheme_name, func in schemes.items():
                 result = func(request)
+                while asyncio.iscoroutine(result):
+                    result = await result
                 if result is self.no_value:
                     return self.no_value
                 token_info[scheme_name] = result
@@ -315,98 +333,108 @@ class AbstractSecurityHandlerFactory(abc.ABC):
 
     def _need_to_add_context_or_scopes(self, func):
         arguments, has_kwargs = inspect_function_arguments(func)
-        need_context = self.pass_context_arg_name and (
-            has_kwargs or self.pass_context_arg_name in arguments)
+        need_context = self.context_kw in arguments
         need_required_scopes = has_kwargs or self.required_scopes_kw in arguments
         return need_context, need_required_scopes
 
     def _generic_check(self, func, exception_msg):
-        need_to_add_context, need_to_add_required_scopes = self._need_to_add_context_or_scopes(
-            func)
+        (
+            need_to_add_context,
+            need_to_add_required_scopes,
+        ) = self._need_to_add_context_or_scopes(func)
 
-        def wrapper(request, *args, required_scopes=None):
+        async def wrapper(request, *args, required_scopes=None):
             kwargs = {}
             if need_to_add_context:
-                kwargs[self.pass_context_arg_name] = request.context
+                kwargs[self.context_kw] = request.context
             if need_to_add_required_scopes:
                 kwargs[self.required_scopes_kw] = required_scopes
             token_info = func(*args, **kwargs)
+            while asyncio.iscoroutine(token_info):
+                token_info = await token_info
             if token_info is self.no_value:
                 return self.no_value
             if token_info is None:
-                raise OAuthResponseProblem(
-                    description=exception_msg, token_response=None)
+                raise OAuthResponseProblem(detail=exception_msg, token_response=None)
             return token_info
 
         return wrapper
 
     def check_bearer_token(self, token_info_func):
-        return self._generic_check(token_info_func, 'Provided token is not valid')
+        return self._generic_check(token_info_func, "Provided token is not valid")
 
     def check_basic_auth(self, basic_info_func):
-        return self._generic_check(basic_info_func, 'Provided authorization is not valid')
+        return self._generic_check(
+            basic_info_func, "Provided authorization is not valid"
+        )
 
     def check_api_key(self, api_key_info_func):
-        return self._generic_check(api_key_info_func, 'Provided apikey is not valid')
+        return self._generic_check(api_key_info_func, "Provided apikey is not valid")
 
     def check_oauth_func(self, token_info_func, scope_validate_func):
         get_token_info = self._generic_check(
-            token_info_func, 'Provided token is not valid')
+            token_info_func, "Provided token is not valid"
+        )
         need_to_add_context, _ = self._need_to_add_context_or_scopes(
-            scope_validate_func)
+            scope_validate_func
+        )
 
-        def wrapper(request, token, required_scopes):
-            token_info = get_token_info(
-                request, token, required_scopes=required_scopes)
+        async def wrapper(request, token, required_scopes):
+            token_info = await get_token_info(
+                request, token, required_scopes=required_scopes
+            )
 
             # Fallback to 'scopes' for backward compatibility
-            token_scopes = token_info.get(
-                'scope', token_info.get('scopes', ''))
+            token_scopes = token_info.get("scope", token_info.get("scopes", ""))
 
             kwargs = {}
             if need_to_add_context:
-                kwargs[self.pass_context_arg_name] = request.context
-            validation = scope_validate_func(
-                required_scopes, token_scopes, **kwargs)
+                kwargs[self.context_kw] = request.context
+            validation = scope_validate_func(required_scopes, token_scopes, **kwargs)
+            while asyncio.iscoroutine(validation):
+                validation = await validation
             if not validation:
                 raise OAuthScopeProblem(
-                    description='Provided token doesn\'t have the required scope',
+                    detail="Provided token doesn't have the required scope",
                     required_scopes=required_scopes,
-                    token_scopes=token_scopes
+                    token_scopes=token_scopes,
                 )
 
             return token_info
+
         return wrapper
 
     @classmethod
-    def verify_security(cls, auth_funcs, function):
-        @functools.wraps(function)
-        def wrapper(request):
+    def verify_security(cls, auth_funcs):
+        async def verify_fn(request):
             token_info = cls.no_value
             errors = []
             for func in auth_funcs:
                 try:
                     token_info = func(request)
+                    while asyncio.iscoroutine(token_info):
+                        token_info = await token_info
                     if token_info is not cls.no_value:
                         break
                 except Exception as err:
                     errors.append(err)
 
-            if token_info is cls.no_value:
+            else:
                 if errors != []:
                     cls._raise_most_specific(errors)
                 else:
                     logger.info("... No auth provided. Aborting with 401.")
-                    raise OAuthProblem(
-                        description='No authorization token provided')
+                    raise OAuthProblem(detail="No authorization token provided")
 
-            # Fallback to 'uid' for backward compatibility
-            request.context['user'] = token_info.get(
-                'sub', token_info.get('uid'))
-            request.context['token_info'] = token_info
-            return function(request)
+            request.context.update(
+                {
+                    # Fallback to 'uid' for backward compatibility
+                    "user": token_info.get("sub", token_info.get("uid")),
+                    "token_info": token_info,
+                }
+            )
 
-        return wrapper
+        return verify_fn
 
     @staticmethod
     def _raise_most_specific(exceptions: t.List[Exception]) -> None:
@@ -428,7 +456,7 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         # We only use status code attributes from exceptions
         # We use 600 as default because 599 is highest valid status code
         status_to_exc = {
-            getattr(exc, 'code', getattr(exc, 'status', 600)): exc
+            getattr(exc, "status_code", getattr(exc, "status", 600)): exc
             for exc in exceptions
         }
         if 403 in status_to_exc:
@@ -439,7 +467,6 @@ class AbstractSecurityHandlerFactory(abc.ABC):
             lowest_status_code = min(status_to_exc)
             raise status_to_exc[lowest_status_code]
 
-    @abc.abstractmethod
     def get_token_info_remote(self, token_info_url):
         """
         Return a function which will call `token_info_url` to retrieve token info.
@@ -451,3 +478,16 @@ class AbstractSecurityHandlerFactory(abc.ABC):
         :type token_info_url: str
         :rtype: types.FunctionType
         """
+
+        async def wrapper(token):
+            if self.client is None:
+                self.client = httpx.AsyncClient()
+            headers = {"Authorization": f"Bearer {token}"}
+            token_request = await self.client.get(
+                token_info_url, headers=headers, timeout=5
+            )
+            if token_request.status_code != 200:
+                return
+            return token_request.json()
+
+        return wrapper
